# -*- coding: utf-8 -*-
"""
bulk_grades Django application initialization.
"""

from django.apps import AppConfig


class BulkGradesConfig(AppConfig):
    """
    Configuration for the bulk_grades Django application.
    """

    name = 'bulk_grades'
    plugin_app = {
        'url_config': {
            'lms.djangoapp': {
                'namespace': u'bulk_grades',
                'regex': u'^api/',
                'relative_path': u'urls',
            },
        },
        u'settings_config': {
            u'lms.djangoapp': {
                u'common': {'relative_path': u'settings.common'},
                u'production': {'relative_path': u'settings.production'},
            },
        },
    }



# -*- coding: utf-8 -*-
"""
URLs for bulk_grades.
"""

from django.conf import settings
from django.conf.urls import url

from . import views

urlpatterns = [
    url(
        r'^bulk_grades/course/{}/$'.format(settings.COURSE_ID_PATTERN),
        views.GradeImportExport.as_view(),
        name='bulk_grades'
    ),
    url(
        r'^bulk_grades/course/{}/history/$'.format(settings.COURSE_ID_PATTERN),
        views.GradeOperationHistoryView.as_view(),
        name='bulk_grades.history'
    ),
    url(
        r'^bulk_grades/course/{}/intervention/$'.format(settings.COURSE_ID_PATTERN),
        views.InterventionsExport.as_view(),
        name='interventions'
    ),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.22 on 2019-07-03 15:26


import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bulk_grades', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='scoreoverrider',
            name='module',
            field=models.ForeignKey(db_constraint=False, on_delete=django.db.models.deletion.CASCADE, to='courseware.StudentModule'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-05-24 17:00


import django.utils.timezone
import model_utils.fields
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('courseware', '__first__'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='ScoreOverrider',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('created', models.DateTimeField(auto_now_add=True, db_index=True)),
                ('module', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='courseware.StudentModule')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



"""
CSV import/export API for grades.
"""

import datetime
import logging

from django.http import HttpResponseForbidden, JsonResponse, StreamingHttpResponse
from django.views.generic import View

from . import api

log = logging.getLogger(__name__)


class GradeOnlyExport(View):
    """
    CSV Export of grade information only. To be used by both bulk grade export and interventions.
    """

    def __init__(self, **kwargs):
        """
        Configure initial state.
        """
        super(GradeOnlyExport, self).__init__(**kwargs)
        self.processor = None
        self.extra_filename = ''

    def get_export_iterator(self, request):
        """
        Return an iterator appropriate for a streaming response.
        """
        return []

    def initialize_processor(self, request, course_id):
        """
        Abstract method to initialize processor particular to the class.
        """

    def dispatch(self, request, course_id, *args, **kwargs):  # pylint: disable=arguments-differ
        """
        Dispatch django request.
        """
        self.initialize_processor(request, course_id)
        return super(GradeOnlyExport, self).dispatch(request, course_id, *args, **kwargs)

    def get(self, request, course_id, *args, **kwargs):
        """
        Export grades in CSV format.

        GET arguments:
        track: name of enrollment mode
        cohort: name of cohort
        subsection: block id of graded subsection
        """
        iterator = self.get_export_iterator(request)
        filename = [course_id]

        if self.extra_filename:
            filename.append(self.extra_filename)
        filename.append(datetime.datetime.utcnow().isoformat())

        response = StreamingHttpResponse(iterator, content_type='text/csv')
        response['Content-Disposition'] = 'attachment; filename="%s.csv"' % '-'.join(filename)

        log.info('Exporting %s CSV for %s', course_id, self.__class__)
        return response


class GradeImportExport(GradeOnlyExport):
    """
    CSV Grade import/export view.
    """

    def post(self, request, course_id, *args, **kwargs):
        """
        Import grades from a CSV file.
        """
        result_id = request.POST.get('result_id', None)
        if result_id:
            results = self.processor.get_deferred_result(result_id)
            if results.ready():
                data = results.get()
                log.info('Got results from celery %r', data)
            else:
                data = {'waiting': True, 'result_id': result_id}
                log.info('Still waiting for %s', result_id)
        else:
            the_file = request.FILES['csv']
            self.processor.process_file(the_file, autocommit=True)
            data = self.processor.status()
            log.info('Processed file %s for %s -> %s saved, %s processed, %s error. (async=%s)',
                     the_file.name,
                     course_id,
                     data.get('saved', 0),
                     data.get('total', 0),
                     len(data.get('error_rows', [])),
                     data.get('waiting', False))
        return JsonResponse(data)

    def get_export_iterator(self, request):
        """
        Create an iterator for exporting grade data.
        """
        return self.processor.get_iterator(error_data=bool(request.GET.get('error_id', '')))

    # pylint: disable=inconsistent-return-statements
    def initialize_processor(self, request, course_id):
        """
        Initialize GradeCSVProcessor.
        """
        operation_id = request.GET.get('error_id', '')
        if operation_id:
            self.processor = api.GradeCSVProcessor.load(operation_id)
            if self.processor.course_id != course_id:
                return HttpResponseForbidden()
            self.extra_filename = 'graded-results'
        else:
            assignment_grade_max = request.GET.get('assignmentGradeMax')
            assignment_grade_min = request.GET.get('assignmentGradeMin')
            course_grade_min = request.GET.get('courseGradeMin')
            course_grade_max = request.GET.get('courseGradeMax')
            self.processor = api.GradeCSVProcessor(
                course_id=course_id,
                user_id=request.user.id,
                track=request.GET.get('track'),
                cohort=request.GET.get('cohort'),
                subsection=request.GET.get('assignment'),
                assignment_type=request.GET.get('assignmentType'),
                subsection_grade_max=(float(assignment_grade_max)
                                      if assignment_grade_max else None),
                subsection_grade_min=(float(assignment_grade_min)
                                      if assignment_grade_min else None),
                course_grade_min=(float(course_grade_min) if course_grade_min else None),
                course_grade_max=(float(course_grade_max) if course_grade_max else None),
            )


class GradeOperationHistoryView(View):
    """
    Collection View for history of grade override file uploads.
    """

    def get(self, request, course_id):
        """
        Get all previous times grades have been overwritten for this course.
        """
        processor = api.GradeCSVProcessor(
            course_id=course_id,
            _user=request.user
        )
        history = processor.get_committed_history()
        return JsonResponse(history, safe=False)


class InterventionsExport(GradeOnlyExport):
    """
    Interventions export view.
    """

    extra_filename = 'intervention'

    def get_export_iterator(self, request):
        """
        Create an iterator for exporting intervention data.
        """
        return self.processor.get_iterator()

    def initialize_processor(self, request, course_id):
        """
        Initialize InterventionCSVProcessor.
        """
        assignment_grade_max = request.GET.get('assignmentGradeMax')
        assignment_grade_min = request.GET.get('assignmentGradeMin')
        course_grade_min = request.GET.get('courseGradeMin')
        course_grade_max = request.GET.get('courseGradeMax')

        self.processor = api.InterventionCSVProcessor(
            course_id=course_id,
            _user=request.user,
            cohort=request.GET.get('cohort'),
            subsection=request.GET.get('assignment'),
            assignment_type=request.GET.get('assignmentType'),
            subsection_grade_min=(float(assignment_grade_min)
                                  if assignment_grade_min else None),
            subsection_grade_max=(float(assignment_grade_max)
                                  if assignment_grade_max else None),
            course_grade_min=(float(course_grade_min) if course_grade_min else None),
            course_grade_max=(float(course_grade_max) if course_grade_max else None)
        )
